(self.webpackChunk_danielfrg_illusionist=self.webpackChunk_danielfrg_illusionist||[]).push([[87],{83087:(s,j,e)=>{var l={"./apl/apl.js":29483,"./asciiarmor/asciiarmor.js":95637,"./asn.1/asn.1.js":97922,"./asterisk/asterisk.js":31881,"./brainfuck/brainfuck.js":9460,"./clike/clike.js":99762,"./clojure/clojure.js":63693,"./cmake/cmake.js":33160,"./cobol/cobol.js":65542,"./coffeescript/coffeescript.js":90044,"./commonlisp/commonlisp.js":29073,"./crystal/crystal.js":76149,"./css/css.js":36629,"./cypher/cypher.js":88441,"./d/d.js":49570,"./dart/dart.js":20842,"./diff/diff.js":1715,"./django/django.js":75288,"./dockerfile/dockerfile.js":75258,"./dtd/dtd.js":48668,"./dylan/dylan.js":25184,"./ebnf/ebnf.js":84134,"./ecl/ecl.js":60427,"./eiffel/eiffel.js":20370,"./elm/elm.js":78919,"./erlang/erlang.js":76392,"./factor/factor.js":27566,"./fcl/fcl.js":15300,"./forth/forth.js":88859,"./fortran/fortran.js":44874,"./gas/gas.js":28902,"./gfm/gfm.js":42425,"./gherkin/gherkin.js":85546,"./go/go.js":94750,"./groovy/groovy.js":53123,"./haml/haml.js":20147,"./handlebars/handlebars.js":62101,"./haskell-literate/haskell-literate.js":28510,"./haskell/haskell.js":18176,"./haxe/haxe.js":19889,"./htmlembedded/htmlembedded.js":79820,"./htmlmixed/htmlmixed.js":16531,"./http/http.js":98411,"./idl/idl.js":41797,"./javascript/javascript.js":96876,"./jinja2/jinja2.js":74670,"./jsx/jsx.js":71548,"./julia/julia.js":68059,"./livescript/livescript.js":14373,"./lua/lua.js":57745,"./markdown/markdown.js":49047,"./mathematica/mathematica.js":93672,"./mbox/mbox.js":94377,"./mirc/mirc.js":94268,"./mllike/mllike.js":52587,"./modelica/modelica.js":24333,"./mscgen/mscgen.js":68810,"./mumps/mumps.js":12875,"./nginx/nginx.js":10131,"./nsis/nsis.js":42313,"./ntriples/ntriples.js":82954,"./octave/octave.js":15734,"./oz/oz.js":44323,"./pascal/pascal.js":23057,"./pegjs/pegjs.js":31472,"./perl/perl.js":10024,"./php/php.js":36702,"./pig/pig.js":54763,"./powershell/powershell.js":97196,"./properties/properties.js":7999,"./protobuf/protobuf.js":72360,"./pug/pug.js":75354,"./puppet/puppet.js":20673,"./python/python.js":5321,"./q/q.js":97713,"./r/r.js":85420,"./rpm/rpm.js":4344,"./rst/rst.js":22968,"./ruby/ruby.js":80853,"./rust/rust.js":90911,"./sas/sas.js":16229,"./sass/sass.js":81201,"./scheme/scheme.js":42372,"./shell/shell.js":31149,"./sieve/sieve.js":49311,"./slim/slim.js":32486,"./smalltalk/smalltalk.js":66690,"./smarty/smarty.js":52889,"./solr/solr.js":18715,"./soy/soy.js":61527,"./sparql/sparql.js":63982,"./spreadsheet/spreadsheet.js":63138,"./sql/sql.js":54086,"./stex/stex.js":80785,"./stylus/stylus.js":60742,"./swift/swift.js":37355,"./tcl/tcl.js":25664,"./textile/textile.js":43897,"./tiddlywiki/tiddlywiki.js":91046,"./tiki/tiki.js":17988,"./toml/toml.js":47617,"./tornado/tornado.js":66043,"./troff/troff.js":20800,"./ttcn-cfg/ttcn-cfg.js":68923,"./ttcn/ttcn.js":71105,"./turtle/turtle.js":73556,"./twig/twig.js":54702,"./vb/vb.js":12660,"./vbscript/vbscript.js":98073,"./velocity/velocity.js":15912,"./verilog/verilog.js":63249,"./vhdl/vhdl.js":53140,"./vue/vue.js":2426,"./wast/wast.js":8632,"./webidl/webidl.js":11015,"./xml/xml.js":29589,"./xquery/xquery.js":61484,"./yacas/yacas.js":65278,"./yaml-frontmatter/yaml-frontmatter.js":98554,"./yaml/yaml.js":53631,"./z80/z80.js":22245};function t(s){var j=r(s);return e(j)}function r(s){if(!e.o(l,s)){var j=new Error("Cannot find module '"+s+"'");throw j.code="MODULE_NOT_FOUND",j}return l[s]}t.keys=function(){return Object.keys(l)},t.resolve=r,s.exports=t,t.id=83087}}]);
//# sourceMappingURL=87.illusionist-embed.js.map